@@ -1,3 +1,18 @@
+/***************************************************************************
+    This file is part of the CheeseTronic Music Tools
+    url                  : http://reduz.com.ar/cheesetronic
+    copyright            : (C) 2003 by Juan Linietsky
+    email                : coding@reduz.com.ar
+ ***************************************************************************/
+
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
 /***************************************************************************
                           player_data_effects.cpp  -  description
                              -------------------
@@ -60,12 +75,15 @@ void Player_Data::do_vibrato(int p_track,bool fine) {
 
 	if (!fine) temp<<=2;
 
-        if (control.channel[p_track].vibrato_position>=0) {
+	for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-                control.channel[p_track].period=control.channel[p_track].aux_period+temp;
-        } else {
-	
-                control.channel[p_track].period=control.channel[p_track].aux_period-temp;
+		if (control.channel[p_track].vibrato_position>=0) {
+
+			control.channel[p_track].layers[i].period=control.channel[p_track].layers[i].aux_period+temp;
+		} else {
+
+			control.channel[p_track].layers[i].period=control.channel[p_track].layers[i].aux_period-temp;
+		}
 	}
 
         if (!song->variables.old_effects || control.ticks_counter) control.channel[p_track].vibrato_position+=control.channel[p_track].vibrato_speed;
@@ -81,17 +99,21 @@ void Player_Data::do_pitch_slide_down(int p_track,Uint8 inf) {
 
         hi=inf>>4;
         lo=inf&0xf;
-		
-        if (hi==0xf) {
 
-                if (!control.ticks_counter) control.channel[p_track].aux_period+=(Uint16)lo<<2;
-        } else if (hi==0xe) {
+	for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-                if (!control.ticks_counter) control.channel[p_track].aux_period+=lo;
-        } else {
 
-                if (control.ticks_counter) control.channel[p_track].aux_period+=(Uint16)inf<<2;
-        }
+		if (hi==0xf) {
+
+			if (!control.ticks_counter) control.channel[p_track].layers[i].aux_period+=(Uint16)lo<<2;
+		} else if (hi==0xe) {
+
+			if (!control.ticks_counter) control.channel[p_track].layers[i].aux_period+=lo;
+		} else {
+
+			if (control.ticks_counter) control.channel[p_track].layers[i].aux_period+=(Uint16)inf<<2;
+		}
+	}
 }
 
 void Player_Data::do_pitch_slide_up(int p_track,Uint8 inf) {
@@ -103,17 +125,20 @@ void Player_Data::do_pitch_slide_up(int p_track,Uint8 inf) {
 
         hi=inf>>4;
         lo=inf&0xf;
-		
-        if (hi==0xf) {
 
-                if (!control.ticks_counter) control.channel[p_track].aux_period-=(Uint16)lo<<2;
-        } else if (hi==0xe) {
+	for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-                if (!control.ticks_counter) control.channel[p_track].aux_period-=lo;
-        } else {
+		if (hi==0xf) {
 
-                if (control.ticks_counter) control.channel[p_track].aux_period-=(Uint16)inf<<2;
-        }
+			if (!control.ticks_counter) control.channel[p_track].layers[i].aux_period-=(Uint16)lo<<2;
+		} else if (hi==0xe) {
+
+			if (!control.ticks_counter) control.channel[p_track].layers[i].aux_period-=lo;
+		} else {
+
+			if (control.ticks_counter) control.channel[p_track].layers[i].aux_period-=(Uint16)inf<<2;
+		}
+	}
 }
 
 void Player_Data::do_pitch_slide_to_note(int p_track) {
@@ -123,27 +148,30 @@ void Player_Data::do_pitch_slide_to_note(int p_track) {
 
                 /* We have to slide a->period towards a->wantedperiod, compute the
                    difference between those two values */
-                dist=control.channel[p_track].period-control.channel[p_track].slide_to_period;
+		for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-            /* if they are equal or if portamentospeed is too big... */
-                if ((!dist)||((control.channel[p_track].portamento_speed<<2)>abs(dist))) {
-                        /* ... make tmpperiod equal tperiod */
-                        control.channel[p_track].aux_period=control.channel[p_track].period=control.channel[p_track].slide_to_period;
-                } else {
+			dist=control.channel[p_track].layers[i].period-control.channel[p_track].layers[i].slide_to_period;
 
-			if (dist>0) {
+		/* if they are equal or if portamentospeed is too big... */
+			if ((!dist)||((control.channel[p_track].portamento_speed<<2)>abs(dist))) {
+				/* ... make tmpperiod equal tperiod */
+				control.channel[p_track].layers[i].aux_period=control.channel[p_track].layers[i].period=control.channel[p_track].layers[i].slide_to_period;
+			} else {
 
-                        	control.channel[p_track].aux_period-=control.channel[p_track].portamento_speed<<2;
-                        	control.channel[p_track].period-=control.channel[p_track].portamento_speed<<2; /* dist>0 slide up */
-	                } else {
-        	                control.channel[p_track].aux_period+=control.channel[p_track].portamento_speed<<2;
-                	        control.channel[p_track].period+=control.channel[p_track].portamento_speed<<2; /* dist<0 slide down */
+				if (dist>0) {
+
+					control.channel[p_track].layers[i].aux_period-=control.channel[p_track].portamento_speed<<2;
+					control.channel[p_track].layers[i].period-=control.channel[p_track].portamento_speed<<2; /* dist>0 slide up */
+				} else {
+					control.channel[p_track].layers[i].aux_period+=control.channel[p_track].portamento_speed<<2;
+					control.channel[p_track].layers[i].period+=control.channel[p_track].portamento_speed<<2; /* dist<0 slide down */
+				}
 			}
 		}
-
         } else {
 
-                control.channel[p_track].aux_period=control.channel[p_track].period;
+		for (int i=0;i<Instrument::MAX_LAYERS;i++)
+	                control.channel[p_track].layers[i].aux_period=control.channel[p_track].layers[i].period;
 	}
 }
 
@@ -166,7 +194,9 @@ void Player_Data::do_tremor(int p_track) {
         off=(inf&0xf);
 
         control.channel[p_track].tremor_position%=(on+off);
-        control.channel[p_track].volume=(control.channel[p_track].tremor_position<on)?control.channel[p_track].aux_volume:0;
+
+	for (int i=0;i<Instrument::MAX_LAYERS;i++)
+	        control.channel[p_track].layers[i].volume=(control.channel[p_track].tremor_position<on)?control.channel[p_track].layers[i].aux_volume:0;
         control.channel[p_track].tremor_position++;
 }
 
@@ -182,24 +212,26 @@ void Player_Data::do_pan_slide(int p_track) {
 
         lo=inf&0xf;
         hi=inf>>4;
+	for (int i=0;i<Instrument::MAX_LAYERS;i++) {
+		pan=(control.channel[p_track].layers[i].panning==PAN_SURROUND)?PAN_CENTER:control.channel[p_track].layers[i].panning;
 
-        pan=(control.channel[p_track].panning==PAN_SURROUND)?PAN_CENTER:control.channel[p_track].panning;
-
-        if (!hi)
-                pan+=lo<<2;
-        else
-          if (!lo) {
-                pan-=hi<<2;
-        } else
-          if (hi==0xf) {
-                if (!control.ticks_counter) pan+=lo<<2;
-        } else
-          if (lo==0xf) {
-                if (!control.ticks_counter) pan-=hi<<2;
-        }
-	//this sets both chan & voice paning
-        control.channel[p_track].panning=(pan<PAN_LEFT)?PAN_LEFT:(pan>PAN_RIGHT?PAN_RIGHT:pan);
-	control.channel[p_track].channel_panning=control.channel[p_track].panning;
+		if (!hi)
+			pan+=lo<<2;
+		else
+		if (!lo) {
+			pan-=hi<<2;
+		} else
+		if (hi==0xf) {
+			if (!control.ticks_counter) pan+=lo<<2;
+		} else
+		if (lo==0xf) {
+			if (!control.ticks_counter) pan-=hi<<2;
+		}
+		//this sets both chan & voice paning
+		control.channel[p_track].layers[i].panning=(pan<PAN_LEFT)?PAN_LEFT:(pan>PAN_RIGHT?PAN_RIGHT:pan);
+	}
+	//this is for compatibility, i guess it doesnt really matter.
+	control.channel[p_track].channel_panning=control.channel[p_track].layers[0].panning;
 }
 
 void Player_Data::do_volume_slide(int p_track,int inf) {
@@ -208,29 +240,32 @@ void Player_Data::do_volume_slide(int p_track,int inf) {
 
 	lo=inf&0xf;
 	hi=inf>>4;
-			
-	if (!lo) {
 
-		if ((control.ticks_counter)) control.channel[p_track].aux_volume+=hi;
+	for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-	} else if (!hi) {
+		if (!lo) {
 
-		if ((control.ticks_counter)) control.channel[p_track].aux_volume-=lo;
+			if ((control.ticks_counter)) control.channel[p_track].layers[i].aux_volume+=hi;
 
-	} else if (lo==0xf) {
+		} else if (!hi) {
 
-		if (!control.ticks_counter) control.channel[p_track].aux_volume+=(hi?hi:0xf);
-	} else if (hi==0xf) {
+			if ((control.ticks_counter)) control.channel[p_track].layers[i].aux_volume-=lo;
 
-		if (!control.ticks_counter) control.channel[p_track].aux_volume-=(lo?lo:0xf);
-	} else return;
+		} else if (lo==0xf) {
 
-	if (control.channel[p_track].aux_volume<0) {
+			if (!control.ticks_counter) control.channel[p_track].layers[i].aux_volume+=(hi?hi:0xf);
+		} else if (hi==0xf) {
 
-		control.channel[p_track].aux_volume=0;
-	} else if (control.channel[p_track].aux_volume>64) {
+			if (!control.ticks_counter) control.channel[p_track].layers[i].aux_volume-=(lo?lo:0xf);
+		} else continue;
 
-		control.channel[p_track].aux_volume=64;
+		if (control.channel[p_track].layers[i].aux_volume<0) {
+
+			control.channel[p_track].layers[i].aux_volume=0;
+		} else if (control.channel[p_track].layers[i].aux_volume>64) {
+
+			control.channel[p_track].layers[i].aux_volume=64;
+		}
 	}
 }
 
@@ -292,18 +327,20 @@ void Player_Data::do_tremolo(int p_track) {
         temp*=control.channel[p_track].tremolo_depth;
         temp>>=7;
 
+	for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
 
-        if (control.channel[p_track].tremolo_position>=0) {
+		if (control.channel[p_track].tremolo_position>=0) {
 
 
-                control.channel[p_track].volume=control.channel[p_track].aux_volume+temp;
-                if (control.channel[p_track].volume>64) control.channel[p_track].volume=64;
-        } else {
+			control.channel[p_track].layers[i].volume=control.channel[p_track].layers[i].aux_volume+temp;
+			if (control.channel[p_track].layers[i].volume>64) control.channel[p_track].layers[i].volume=64;
+		} else {
 
-		control.channel[p_track].volume=control.channel[p_track].aux_volume-temp;
-                if (control.channel[p_track].volume<0) control.channel[p_track].volume=0;
-        }
+			control.channel[p_track].layers[i].volume=control.channel[p_track].layers[i].aux_volume-temp;
+			if (control.channel[p_track].layers[i].volume<0) control.channel[p_track].layers[i].volume=0;
+		}
+	}
 
         /*if (control.ticks_counter)*/ control.channel[p_track].tremolo_position+=control.channel[p_track].tremolo_speed;
 
@@ -321,29 +358,32 @@ void Player_Data::do_arpegio(int p_track) {
 	}
 
 	dat=control.channel[p_track].arpegio_info;
-				
+
 	if (dat) {
 
 		switch (control.ticks_counter%3) {
-			
+
 			case 1: {
 
 	                        note+=(dat>>4);
 
 			} break;
 			case 2: {
-	
+
                                 note+=(dat&0xf);
 			} break;
 		}
 
-		if (song->variables.use_linear_slides) {
+		for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-			control.channel[p_track].period=control.channel[p_track].aux_period-abs(get_period((Uint16)46,0)-get_period((Uint16)44,0))*note;
-		} else {
+			if (song->variables.use_linear_slides) {
 
-			control.channel[p_track].period=get_period( (((Uint16)control.channel[p_track].note+control.channel[p_track].sample_ptr->data.note_offset)+note)<<1,control.channel[p_track].sample_ptr->data.finetune);
-                }
+				control.channel[p_track].layers[i].period=control.channel[p_track].layers[i].aux_period-abs(get_period((Uint16)46,0,song->variables.use_linear_slides)-get_period((Uint16)44,0,song->variables.use_linear_slides))*note;
+			} else { //@TODO fix this in some non-hacking way
+
+				control.channel[p_track].layers[i].period=get_period( (((Uint16)control.channel[p_track].layers[i].note)+note)<<1,0,song->variables.use_linear_slides);
+			}
+		}
         	control.channel[p_track].has_own_period=true;
         }
 
@@ -369,41 +409,47 @@ void Player_Data::do_retrig(int p_track) {
                 if ( !control.channel[p_track].retrig_counter ) {
                         /* when retrig counter reaches 0, reset counter and restart the
                            sample */
-                        if (control.channel[p_track].kick!=KICK_NOTE) control.channel[p_track].kick=KICK_NOTEOFF;
 			control.channel[p_track].retrig_counter=control.channel[p_track].retrig_speed;
 
+			for (int i=0;i<Instrument::MAX_LAYERS;i++) {
+				//if (control.channel[p_track].layers[i].kick!=KICK_NOTE)
+				//	control.channel[p_track].layers[i].kick=KICK_NOTEOFF; //
 
-                        if ((control.ticks_counter)/*||(pf->flags&UF_S3MSLIDES)*/) {
-                                switch (control.channel[p_track].retrig_volslide) {
-                                        case 1:
-                                        case 2:
-                                        case 3:
-                                        case 4:
-                                        case 5:
-                                                control.channel[p_track].aux_volume-=(1<<(control.channel[p_track].retrig_volslide-1));
-                                                break;
-                                        case 6:
-                                                control.channel[p_track].aux_volume=(2*control.channel[p_track].aux_volume)/3;
-                                                break;
-                                        case 7:
-                                                control.channel[p_track].aux_volume>>=1;
-                                                break;
-                                        case 9:
-                                        case 0xa:
-                                        case 0xb:
-                                        case 0xc:
-                                        case 0xd:
-                                                control.channel[p_track].aux_volume+=(1<<(control.channel[p_track].retrig_volslide-9));
-                                                break;
-                                        case 0xe:
-                                                control.channel[p_track].aux_volume=(3*control.channel[p_track].aux_volume)>>1;
-                                                break;
-                                        case 0xf:
-                                                control.channel[p_track].aux_volume=control.channel[p_track].aux_volume<<1;
-                                                break;
-                                }
-                                if (control.channel[p_track].aux_volume<0) control.channel[p_track].aux_volume=0;
-                                else if (control.channel[p_track].aux_volume>64) control.channel[p_track].aux_volume=64;
+				control.channel[p_track].layers[i].kick=KICK_NOTE;
+
+
+				if ((control.ticks_counter)/*||(pf->flags&UF_S3MSLIDES)*/) {
+					switch (control.channel[p_track].retrig_volslide) {
+						case 1:
+						case 2:
+						case 3:
+						case 4:
+						case 5:
+							control.channel[p_track].layers[i].aux_volume-=(1<<(control.channel[p_track].retrig_volslide-1));
+							break;
+						case 6:
+							control.channel[p_track].layers[i].aux_volume=(2*control.channel[p_track].layers[i].aux_volume)/3;
+							break;
+						case 7:
+							control.channel[p_track].layers[i].aux_volume>>=1;
+							break;
+						case 9:
+						case 0xa:
+						case 0xb:
+						case 0xc:
+						case 0xd:
+							control.channel[p_track].layers[i].aux_volume+=(1<<(control.channel[p_track].retrig_volslide-9));
+							break;
+						case 0xe:
+							control.channel[p_track].layers[i].aux_volume=(3*control.channel[p_track].layers[i].aux_volume)>>1;
+							break;
+						case 0xf:
+							control.channel[p_track].layers[i].aux_volume=control.channel[p_track].layers[i].aux_volume<<1;
+							break;
+					}
+					if (control.channel[p_track].layers[i].aux_volume<0) control.channel[p_track].layers[i].aux_volume=0;
+					else if (control.channel[p_track].layers[i].aux_volume>64) control.channel[p_track].layers[i].aux_volume=64;
+				}
                         }
                 }
                 control.channel[p_track].retrig_counter--; /* countdown  */
@@ -470,7 +516,10 @@ void Player_Data::do_panbrello(int p_track) {
         temp<<=1;
 	temp+=control.channel[p_track].channel_panning;
 
-        control.channel[p_track].panning=(temp<PAN_LEFT)?PAN_LEFT:(temp>PAN_RIGHT?PAN_RIGHT:temp);
+	for (int i=0;i<Instrument::MAX_LAYERS;i++)
+		if (control.channel[p_track].layers[i].panning!=PAN_SURROUND)
+		        control.channel[p_track].layers[i].panning=(temp<PAN_LEFT)?PAN_LEFT:(temp>PAN_RIGHT?PAN_RIGHT:temp);
+
         control.channel[p_track].panbrello_position+=control.channel[p_track].panbrello_speed;
 }
 
@@ -499,9 +548,13 @@ void Player_Data::do_effect_S(int p_track) {
         switch (c) {
                 case 1: {/* S1x set glissando voice */
 		// this is unsupported in IT!
+
+			//@TODO add something
+  			//control.channel[p_track].chorus_send=inf*0xFF/0xF;
+
                 }break;
                 case 2: /* S2x set finetune */
-		//Also not supported!	
+		//Also not supported!
                         break;
                 case 3: /* S3x set vibrato waveform */
 			if (inf<4) control.channel[p_track].vibrato_type=inf;
@@ -526,13 +579,17 @@ void Player_Data::do_effect_S(int p_track) {
 //			if (pf->panflag) {
                                 if (inf<=8) inf<<=4;
                                 else inf*=17;
-                                control.channel[p_track].panning=control.channel[p_track].channel_panning=inf;
+				control.channel[p_track].channel_panning=inf;
+				for (int i=0;i<Instrument::MAX_LAYERS;i++)
+	                                control.channel[p_track].layers[i].panning=inf;
 //                        }
                 } break;
 
                 case 9: { /* S9x set surround sound */
                         //if (pf->panflag)
-                                control.channel[p_track].panning=control.channel[p_track].channel_panning=PAN_SURROUND;
+
+				for (int i=0;i<Instrument::MAX_LAYERS;i++)
+	                                control.channel[p_track].layers[i].panning=control.channel[p_track].channel_panning=PAN_SURROUND;
                 } break;
                 case 0xA:{ /* SAy set high order sample offset yxx00h */
 
@@ -570,7 +627,8 @@ void Player_Data::do_effect_S(int p_track) {
 
 			if (control.ticks_counter>=inf) {
 
-				control.channel[p_track].aux_volume=0;				
+				for (int i=0;i<Instrument::MAX_LAYERS;i++)
+					control.channel[p_track].layers[i].aux_volume=0;
 			}
                 } break;
                 case 0xD: {/* SDx notedelay */
@@ -651,27 +709,35 @@ void Player_Data::run_volume_column_effects(int p_track) {
 			do_pitch_slide_up(p_track,param<<2);
 		} break;
 		case 'G': {
-		
+
              	        const Uint8 slide_table[]={0,1,4,8,16,32,64,96,128,255};
 			if (param) {
 
 				control.channel[p_track].portamento_speed=slide_table[param];
 			}
 
-			if (control.channel[p_track].period && (control.channel[p_track].old_note<=120)) {
+			if ( control.channel[p_track].old_note<=120) {
 
-				if ( (!control.ticks_counter) && (control.channel[p_track].new_instrument) ){
-				
-					//control.channel[p_track].kick=KICK_NOTE;
-					//control.channel[p_track].sample_start_index=0; // < am i stupid?
-				} else {
+				for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-					control.channel[p_track].kick=(control.channel[p_track].kick==KICK_NOTE)?KICK_ENVELOPE:KICK_NOTHING;
-					do_pitch_slide_to_note(p_track);
-					control.channel[p_track].has_own_period=true;
+					if (!control.channel[p_track].layers[i].period)
+						continue;
+					if ( (!control.ticks_counter) && (control.channel[p_track].layers[i].sample_changed) ){
+
+
+						control.channel[p_track].layers[i].kick=KICK_NOTE;
+						control.channel[p_track].sample_start_index=0;
+
+					} else {
+
+						control.channel[p_track].layers[i].kick=(control.channel[p_track].layers[i].kick==KICK_NOTE)?KICK_ENVELOPE:KICK_NOTHING;
+					}
 				}
 
+				do_pitch_slide_to_note(p_track);
+				control.channel[p_track].has_own_period=true;
 			}
+
 		} break;
 		case 'H': {
 
@@ -679,13 +745,14 @@ void Player_Data::run_volume_column_effects(int p_track) {
 			if (!control.ticks_counter) {
 				if (param&0x0f) control.channel[p_track].vibrato_depth=param;
 			}
-			control.channel[p_track].doing_vibrato=true;			
-                        if (control.external_vibrato) break;			
-			if (control.channel[p_track].period) {
+			control.channel[p_track].doing_vibrato=true;
+                        if (control.external_vibrato) break;
+
+			//if (control.channel[p_track].layers[i].period) {
 
 				do_vibrato(p_track,false);
 				control.channel[p_track].has_own_period=true;
-			}                                		
+			//}
 
 		} break;
 	}
@@ -765,7 +832,7 @@ void Player_Data::run_effects(int p_track) {
 				}
 
        				control.position.force_next_order=next_order;
-					
+
        			} else {
        				// no, probably the user deleted the orderlist.
        				control.play_mode=PLAY_NOTHING;
@@ -781,22 +848,22 @@ void Player_Data::run_effects(int p_track) {
 
 			if (inf) control.channel[p_track].volume_slide_info=inf;
 			else inf=control.channel[p_track].volume_slide_info;
-	
+
 			do_volume_slide(p_track,inf);
 
 		} break;
 		case 'E': {
-		
+
 		        Uint8 inf;
-			
+
 			inf=control.channel[p_track].current_parameter;
 			do_pitch_slide_down(p_track,inf);
 
 		} break;
 		case 'F': {
-		
+
 		        Uint8 inf;
-			
+
 			inf=control.channel[p_track].current_parameter;
 			do_pitch_slide_up(p_track,inf);
 
@@ -808,17 +875,23 @@ void Player_Data::run_effects(int p_track) {
 				control.channel[p_track].portamento_speed=control.channel[p_track].current_parameter;
 			}
 
-			if (control.channel[p_track].period && (control.channel[p_track].old_note<=120)) {
 
-				if ( (!control.ticks_counter) && (control.channel[p_track].new_instrument) ){
-				
+			if ( control.channel[p_track].old_note<=120) {
 
-					control.channel[p_track].kick=KICK_NOTE;
-					control.channel[p_track].sample_start_index=0;
+				for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-				} else {
+					if (!control.channel[p_track].layers[i].period)
+						continue;
+					if ( (!control.ticks_counter) && (control.channel[p_track].layers[i].sample_changed) ){
 
-					control.channel[p_track].kick=(control.channel[p_track].kick==KICK_NOTE)?KICK_ENVELOPE:KICK_NOTHING;
+
+						control.channel[p_track].layers[i].kick=KICK_NOTE;
+						control.channel[p_track].sample_start_index=0;
+
+					} else {
+
+						control.channel[p_track].layers[i].kick=(control.channel[p_track].layers[i].kick==KICK_NOTE)?KICK_ENVELOPE:KICK_NOTHING;
+					}
 				}
 
 				do_pitch_slide_to_note(p_track);
@@ -829,7 +902,7 @@ void Player_Data::run_effects(int p_track) {
       		case 'H': {
 
 			Uint8 dat;
-			
+
 			control.channel[p_track].doing_vibrato=true;
 
 			dat=control.channel[p_track].current_parameter;
@@ -839,13 +912,13 @@ void Player_Data::run_effects(int p_track) {
 				if (dat&0xf0) control.channel[p_track].vibrato_speed=(dat&0xf0)>>2;
 			}
 
-                        if (control.external_vibrato) break;			
-			
-			if (control.channel[p_track].period) {
+                        if (control.external_vibrato) break;
 
-				do_vibrato(p_track,false);
-				control.channel[p_track].has_own_period=true;
-			}                                		
+			//if (control.channel[p_track].period) {
+
+			do_vibrato(p_track,false);
+			control.channel[p_track].has_own_period=true;
+			//}
 
 		} break;
 		case 'I': {
@@ -862,48 +935,59 @@ void Player_Data::run_effects(int p_track) {
 			Uint8 inf ;
 			//explicitslides=1;
 			inf=control.channel[p_track].current_parameter;
-			
+
 			control.channel[p_track].doing_vibrato=true;
 
 
 			if (inf) control.channel[p_track].volume_slide_info=inf;
 			else inf=control.channel[p_track].volume_slide_info;
-	
+
 			do_volume_slide(p_track,inf);
 
                         if (control.external_vibrato) break;						
-			
-			if (control.channel[p_track].period) {
+
+			//if (control.channel[p_track].period) {
 
 				do_vibrato(p_track,false);
 				control.channel[p_track].has_own_period=true;
-			}                                		
+			//}
 
 		} break;
 		case 'L': {
+
+
 			Uint8 inf ;
 			//explicitslides=1;
 			inf=control.channel[p_track].current_parameter;
 
 			if (inf) control.channel[p_track].volume_slide_info=inf;
 			else inf=control.channel[p_track].volume_slide_info;
-	
+
 			do_volume_slide(p_track,inf);
 
-			if (control.channel[p_track].period && (control.channel[p_track].old_note<=120)) {
-				if ( (!control.ticks_counter) && (control.channel[p_track].new_instrument) ){
-				
-					control.channel[p_track].kick=KICK_NOTE;
-					control.channel[p_track].sample_start_index=0;
+			if ( control.channel[p_track].old_note<=120) {
 
-				} else {
+				for (int i=0;i<Instrument::MAX_LAYERS;i++) {
 
-					control.channel[p_track].kick=(control.channel[p_track].kick==KICK_NOTE)?KICK_ENVELOPE:KICK_NOTHING;
+					if (!control.channel[p_track].layers[i].period)
+						continue;
+					if ( (!control.ticks_counter) && (control.channel[p_track].layers[i].sample_changed) ){
+
+
+						control.channel[p_track].layers[i].kick=KICK_NOTE;
+						control.channel[p_track].sample_start_index=0;
+
+					} else {
+
+						control.channel[p_track].layers[i].kick=(control.channel[p_track].layers[i].kick==KICK_NOTE)?KICK_ENVELOPE:KICK_NOTHING;
+					}
 				}
 
 				do_pitch_slide_to_note(p_track);
 				control.channel[p_track].has_own_period=true;
 			}
+
+
 		} break;
 		case 'M': {
 		                control.channel[p_track].channel_volume=control.channel[p_track].current_parameter;
@@ -917,12 +1001,12 @@ void Player_Data::run_effects(int p_track) {
 		case 'O': {
 
 			if (!control.ticks_counter) {
-			
+
 				if (control.channel[p_track].current_parameter) control.channel[p_track].lo_offset=(Uint16)control.channel[p_track].current_parameter<<8;
 				control.channel[p_track].sample_start_index=control.channel[p_track].hi_offset|control.channel[p_track].lo_offset;
 
 				//if ((control.channel[p_track].sample_ptr!=NULL)&&(control.channel[p_track].sample_start_index>control.channel[p_track].sample_ptr->data.size)) {
-					//TODO, O effect				
+					//TODO, O effect
 					//a->start=a->s->flags&(SF_LOOP|SF_BIDI)?a->s->loopstart:a->s->length;
                                 //}
 			}
@@ -991,10 +1075,10 @@ void Player_Data::run_effects(int p_track) {
 				}
 				control.tempo=(temp>255)?255:(temp<0x20?0x20:temp);
 		        }
-		
+
 		} break;
 		case 'U': {
-			
+
 			Uint8 dat;
 
 			dat=control.channel[p_track].current_parameter;
@@ -1004,13 +1088,13 @@ void Player_Data::run_effects(int p_track) {
 				if (dat&0xf0) control.channel[p_track].vibrato_speed=(dat&0xf0)>>2;
 			}
 
-                        if (control.external_vibrato) break;						
+                        if (control.external_vibrato) break;
 
-			if (control.channel[p_track].period) {
+			//if (control.channel[p_track].period) {
 
 				do_vibrato(p_track,true);
 				control.channel[p_track].has_own_period=true;
-			}                                		
+			//}
 		} break;
 		case 'V': {
 
@@ -1023,7 +1107,8 @@ void Player_Data::run_effects(int p_track) {
 		case 'X': {
 			//sets both channel and current
 			control.channel[p_track].channel_panning=control.channel[p_track].current_parameter;
-			control.channel[p_track].panning=control.channel[p_track].current_parameter;
+			for (int i=0;i<Instrument::MAX_LAYERS;i++)
+				control.channel[p_track].layers[i].panning=control.channel[p_track].current_parameter;
 		} break;
 		case 'Y': {
 
@@ -1043,26 +1128,29 @@ void Player_Data::run_effects(int p_track) {
 			}
 
 			//if (pf->panflag)
-			if (control.channel[p_track].panning!=PAN_SURROUND)do_panbrello(p_track);
-		
+			do_panbrello(p_track);
+
 		} break;
 		case 'Z': {
 			//I DO! cuttoff!
 			Uint16 dat=control.channel[p_track].current_parameter;
-			
-			if (dat<0x80) {
-			
-				control.channel[p_track].filter.it_cutoff=dat*2;
-				if (control.channel[p_track].filter.it_cutoff>0x80)
-					control.channel[p_track].filter.it_cutoff++;
-			} else if (dat<0x90) {
-			
-				control.channel[p_track].filter.it_reso=(dat-0x80)*0x10;
-			} else {
-			
-				control.channel[p_track].reverb_send=(dat-0x90)*255/0x6F;
+
+			for (int i=0;i<Instrument::MAX_LAYERS;i++) {
+
+				if (dat<0x80) {
+
+					control.channel[p_track].layers[i].filter.it_cutoff=dat*2;
+					if (control.channel[p_track].layers[i].filter.it_cutoff>0x80)
+						control.channel[p_track].layers[i].filter.it_cutoff++;
+				} else if (dat<0x90) {
+
+					control.channel[p_track].layers[i].filter.it_reso=(dat-0x80)*0x10;
+				} else {
+
+					//control.channel[p_track].reverb_send=(dat-0x90)*255/0x6F;
+				}
 			}
-			
+
 		} break;
 
 	}
@@ -1071,28 +1159,17 @@ void Player_Data::run_effects(int p_track) {
 
 void Player_Data::pre_process_effects() {
 
-//        MP_VOICE *aout;
+
 	int i;
 
         for (i=0;i<PATTERN_WIDTH;i++) {
 
-                //a=&pf->control[mp_channel];
-
-               // if ((aout=a->slave)) {
-                //        a->fadevol=aout->fadevol;
-                 //       a->period=aout->period;
-                 //       if (a->kick==KICK_KEYOFF) a->keyoff=aout->keyoff;
-                //}
-
-                //if (!a->row) continue;
-                //UniSetRow(a->row);
 		control.channel[i].has_own_period=false;
 		control.channel[i].has_own_volume=false;
 		control.channel[i].doing_vibrato=false;
-                //explicitslides=0;
-                //pt_playeffects();
+
 		if (control.ticks_counter<control.speed) {
-	
+
 			run_effects(i);
 			run_volume_column_effects(i);
 		}
@@ -1104,35 +1181,36 @@ void Player_Data::pre_process_effects() {
                  //       else if (a->tmpvolume) a->sliding=explicitslides;
                 //}
 
-                if (!control.channel[i].has_own_period) control.channel[i].period=control.channel[i].aux_period;
-                if (!control.channel[i].has_own_volume) control.channel[i].volume=control.channel[i].aux_volume;
 
-                if ((control.channel[i].sample_ptr!=NULL) && !(song->variables.use_instruments && (control.channel[i].instrument_ptr==NULL))) {
+		if (!control.channel[i].has_own_period)
+			for (int j=0;j<Instrument::MAX_LAYERS;j++)
+				control.channel[i].layers[j].period=control.channel[i].layers[j].aux_period;
 
-			if (song->variables.use_instruments) {
+		if (!control.channel[i].has_own_volume)
+			for (int j=0;j<Instrument::MAX_LAYERS;j++)
+				control.channel[i].layers[j].volume=control.channel[i].layers[j].aux_volume;
 
-                                control.channel[i].output_volume=	
-					(control.channel[i].volume*control.channel[i].sample_ptr->glb_volume*control.channel[i].instrument_ptr->volume.global_amount)/2048;
-				control.channel[i].output_volume=control.channel[i].output_volume*control.channel[i].random_volume_variation/100;
-				
-                        } else {
 
-				control.channel[i].output_volume=
-					(control.channel[i].volume*control.channel[i].sample_ptr->glb_volume)>>4;
-				
+                for (int j=0;j<Instrument::MAX_LAYERS;j++) {
+
+			if ((control.channel[i].layers[j].sample_ptr!=NULL) && (control.channel[i].layers[j].layer_ptr!=NULL)) {
+
+
+				control.channel[i].layers[j].output_volume=
+					(control.channel[i].layers[j].volume*control.channel[i].layers[j].sample_ptr->glb_volume*control.channel[i].layers[j].layer_ptr->volume.global_amount)/2048;
+				control.channel[i].layers[j].output_volume=control.channel[i].layers[j].output_volume*control.channel[i].layers[j].random_volume_variation/100;
+
+
+				if (control.channel[i].layers[j].output_volume>256) {
+
+					control.channel[i].layers[j].output_volume=256;
+
+				} else if (control.channel[i].layers[j].output_volume<0) {
+
+					control.channel[i].layers[j].output_volume=0;
+				}
 			}
+		}
 
-			if (control.channel[i].output_volume>256) {
-				
-				control.channel[i].output_volume=256;
-
-			} else if (control.channel[i].output_volume<0) {
-
-				control.channel[i].output_volume=0;
-			}
-
-			
-               }
         }
-
 }
